import React, { useEffect, useState, useRef } from 'react'
import { Button, Modal, Select, Checkbox, message } from 'antd'
import { Row, Col } from 'antd'

import classnames from 'classnames'
import { useSize } from 'react-hook-size'
import {
	useParams,useHistory
  } from "react-router-dom";

import {
	AudioOutlined,
	SoundOutlined,
	VideoCameraOutlined,
	SettingFilled,
	MenuOutlined,
	UserAddOutlined,
	FundProjectionScreenOutlined,
	PhoneOutlined
} from '@ant-design/icons'

import {
	ViewStyle,
	MainStyle,
	BToolStyle,
	MemberListStyle,
	BToolLeftStyle,
	BToolMidStyle,
	BToolRightStyle,
	MainCenterStyle,
	MainRightStyle,
	LayoutStyle,
	VolumBarWrap
} from './style'



import PeerView from './PeerView/PeerView'

import MemberList from './MemberList/MemberList'

import Detail from './MeetingDetail/MeetingDetail'

import Store from '../../store//index'

const Room =()=>{
	const history = useHistory();
	useEffect(() => {
		if(window.krtcClient){
			const isLogin = JSON.parse(window.atob(JSON.parse(localStorage.getItem(window.krtcClient && window.krtcClient._singleSdk.getClientId()))["kAuthorization"]))["ssoToken"] || null;
			if (!Store.user || !isLogin) {
				history.push("/login");
				return;
			}
		}else{
			history.push("/login");
			return;
		}
		doJoin();
		return ()=>{
			if(window.krtcClient){
				window.krtcClient.exit()
			}
		}
	}, [])
	const {roomId} = useParams();

	//设置内的测试画面指向
	const testVideRef = useRef()

	//中央大屏指向
	const mainRef = useRef()

	//取中央大屏宽度
	const { width, height } = useSize(mainRef)

	//默认初始化参数配置弹框
	// const [visible, setVisible] = useState(true)

	//麦克风 音频等硬件设备选择
	const [visible1, setVisible1] = useState(false)

	//终端列表是否显示
	const [mtListShow, setMtListShow] = useState(false)

	//是否显示会议邀请信息
	const [detailExpand, setDetailExpand] = useState(false)

	//设备相关
	const [audioInputDevices, setAudioInputDevices] = useState([])
	const [audioOutDevices, setAudioOutDevices] = useState([])
	const [videoInputDevices, setVideoInputDevices] = useState([])
	const [meCurrentAudioinputDeviceId, setMeCurrentAudioinputDeviceId] = useState([])
	const [meCurrentAudiooutputDeviceId, setMeCurrentAudiooutputDeviceIdUpdate] = useState([])
	const [meCurrentVideoinputDeviceId, setMeCurrentVideoinputDeviceId] = useState([])
	const [settingOutVideoStream, setSettingOutVideoStream] = useState(null)

	//成员列表
	const [mtlistPeersMap, setMtlistPeersMap] = useState({})

	//终端列表
	const [mtList, setMtList] = useState([])

	//消费对象 里面有视频流
	const [consumersMap, setConsumersMap] = useState({})
	const [shareConsumers, setShareConsumers] = useState({})
	const [vmpConsumers,setVmpConsumers]= useState({})

	//本地视频流 我的
	const [localVideoTrack, setLocalVideoTrack] = useState(null)
	//我的音量
	const [meVolumeValue, setMeVolumeValue] = useState(0)

	//我的
	const [meInfo, setMeInfo] = useState(null)
	//是否哑音
	const [audioMuted, setAudioMuted] = useState(true)

	//是否推送音频
	const [isAudioProducer, setIsAudioProducer] = useState(false)
	//是否推送视频
	const [isVideoProducer, setIsVideoProducer] = useState(false)
	//是否推送双流
	const [isShareProducer, setIsShareProducer] = useState(false)

	const [facemode, setFacemode] = useState(true)

	//当前大屏显示的标识   10-video  10-dualvideo
	const [activePeerKey, setActivePeerKey] = useState('')

	//会议信息
	const [meetingInfo, setMeetingInfo] = useState({})

	//扬声器调试
	const [paused,setPaused] =useState(true)

	//终端画面列表
	const [peerListView, setPeerListView]= useState([])

	const localVideoTrackUpdate =  (event) => {
		console.log(event, '=======localVideoTrackUpdate=======')
		if (event.success) {
			let videoTrack = null
			if (event.data) {
				videoTrack = event.data.videoTrack || null
			}
			setLocalVideoTrack(videoTrack)
		}
	}
	const meVolumeUpdate =  data => {
		setMeVolumeValue(data)
	}
	const mediasConsumersUpdate = data => {
		console.log('===========mediasConsumersUpdate=============', data)
		setTimeout(() => {
			setConsumersMap(data)
		}, 0)
	}
	const mediasShareConsumersUpdate =   data => {
		console.log(data, '=======mediasShareConsumersUpdate=======')
		setTimeout(() => {
			setShareConsumers(data)
		}, 0)
	}
	const mediasVmpConsumersUpdate =   (data) => {
		console.log(data, '=======mediasVmpConsumersUpdate=======')
		setTimeout(() => {
			setVmpConsumers(data)
		}, 0)
	}
	
	const meAudioMutedUpdate =  _audioMuted => {
		console.log(_audioMuted, '=======meAudioMutedUpdate=======')
		setAudioMuted(_audioMuted)
	}
	const mediasProducersUpdate =  _producers => {
		console.log(_producers, '=======mediasProducersUpdate=======')

		setIsVideoProducer(Object.values(_producers).find(p => p.track.kind === 'video'))
		setIsAudioProducer(Object.values(_producers).find(p => p.track.kind === 'audio'))
	}
	const mediasShareProducersUpdate =   _producers => {
		console.log(_producers, '=======mediasShareProducersUpdate=======')
		const b = Object.values(_producers).find(p => p.track.kind === 'video')
		setIsShareProducer(!!b)
	}
	
	//事件监听
	const listenRegister = () => {
		var krtcClient = window.krtcClient

		//本地视频流
		krtcClient.on('localVideoTrackUpdate', localVideoTrackUpdate)

		//成员列表
		krtcClient.on('mtlistPeersUpdate', handleMemberList)
		

		//普通消费者列表
		krtcClient.on('mediasConsumersUpdate', mediasConsumersUpdate)

		//双流消费者列表
		krtcClient.on('mediasShareConsumersUpdate',mediasShareConsumersUpdate)

		//我的声音
		krtcClient.on('meVolumeUpdate',meVolumeUpdate )

		//我的是否否静音
		krtcClient.on('meAudioMutedUpdate',meAudioMutedUpdate )

		//普通生产者
		krtcClient.on('mediasProducersUpdate',mediasProducersUpdate)

		//双流生产者
		krtcClient.on('mediasShareProducersUpdate',mediasShareProducersUpdate)

		//画面合成消费者
		krtcClient.on('mediasVmpConsumersUpdate',mediasVmpConsumersUpdate)

		//监听报错
		krtcClient._singleSdk.on('error',handleErrorEvent)
	}
	const handleErrorEvent = (event) => {
		console.log("skyClient.on  error", event)
      if (event.type == 'enableShareError') {
		  message.warning(event.error.description)
        
      }
	}
	const handleMemberList = (data)=>{
		console.log(data, '=======mtlistPeersUpdate=======')
		setMtlistPeersMap(data)
		let memberList = Object.keys(data)
		.filter(k => k.indexOf('-dualvideo') === -1)
		.filter(k => k.indexOf('-confvmp') === -1)
		.map((k, i) => {
			return {
				...data[k]
			}
		})
		setMtList(memberList)
		setMeInfo(window.krtcClient._singleSdk.getMe())
	}

	//页面初始化
	// useEffect(() => {
	// 	doJoin();
	// 	return ()=>{
	// 		if(window.krtcClient){
	// 			window.krtcClient.exit()
	// 		}
	// 	}
	// }, [])

	//打开关闭设置弹框
	useEffect(() => {
		if (visible1) {
			if (meCurrentVideoinputDeviceId !== '') {
				window.navigator.mediaDevices.getUserMedia({ video: true }).then(stream => {
					setSettingOutVideoStream(stream)
					setTimeout(() => {
						testVideRef && testVideRef.current && (testVideRef.current.srcObject = stream)
					}, 50)
				})
			}
		} else {
			if (meCurrentVideoinputDeviceId !== '') {
				var stream = settingOutVideoStream
				stream &&
					stream.getTracks().forEach(track => {
						track.stop()
					})
			}
		}
	}, [visible1])
	// 入会
	const doJoin = () => {
		const krtcClient = window.krtcClient
		listenRegister()
		if(!krtcClient){return false}
		krtcClient.join({
			roomId: roomId,
			success: function() {
				console.log("入会成功")
				getALLDevices()
				setTimeout(() => {
					krtcClient.enableCamera({
						success: result => {}
					})
				}, 2000)

				setMeInfo(krtcClient._singleSdk.getMe())
				setMeetingInfo(krtcClient._singleSdk.getMeetingInfo())
			},
			error: function(errModal) {
				const { description } = errModal
				message.warning(description)
			}
		})
	}


	//大屏某一宽高比计算
	const getFitRect = (w, h, rate = 480 / 680) => {
		var toH = h
		var toW = Math.round(h / rate)
		if (toW > w) {
			toW = w
			toH = Math.round(toW * rate)
		}
		var re = {
			width: toW,
			height: toH
		}
		return re
	}

	//获取排序成员列表 同时map=>arr
	const getSortPeerList = () => {
		var memberList = Object.keys(mtlistPeersMap)
			.filter(k => k.indexOf('-dualvideo') === -1)
			.filter(k => k.indexOf('-confvmp') === -1)
			.sort((a, b) => parseInt(a) > parseInt(b))
			.map((k, i) => {
				return {
					...mtlistPeersMap[k]
				}
			})
		var dualvideoFullKey = Object.keys(mtlistPeersMap).find(k => k.indexOf('-dualvideo') > -1)
		var vmpvideoFullKey = Object.keys(mtlistPeersMap).find(k => k.indexOf('-confvmp') > -1)
		//画面合成第二
		if(vmpvideoFullKey&&mtlistPeersMap[vmpvideoFullKey]){
			memberList.unshift(mtlistPeersMap[vmpvideoFullKey])
		}
		// 共享最前
		if (dualvideoFullKey && mtlistPeersMap[dualvideoFullKey]) {
			memberList.unshift(mtlistPeersMap[dualvideoFullKey])
		}

		// if (memberList.length > 0) {
		// 	var key = memberList[0].mtId + memberList[0].isDoubleflow ? '-dualvideo' : '-video'
		// } else {
		// }
		memberList = memberList.splice(0,6)
		return memberList
	}

	useEffect(()=>{
		let peerList = getSortPeerList();
		setPeerListView(peerList)
	},[mtlistPeersMap])

	//处理邀请信息展开信息
	const handleDetail = ()=>{
		setMtListShow(false)
		if(detailExpand){
			setDetailExpand(false)
		}else{
			setDetailExpand(true)
		}
	}
	//处理终端列表
	const handleMtList = ()=>{
		setDetailExpand(false)
		if(mtListShow){
			setMtListShow(false)
		}else{
			setMtListShow(true)
		}
	}

	//测试扬声器
	const handleTestBtnClick = ()=>{
		var krtcClient = window.krtcClient
		if (paused) {//是否测试中
			krtcClient.testLoudspeaker({
				playstatus: false,
				deviceId: meCurrentAudiooutputDeviceId,
				success: function () {
					setPaused(!paused)
				}
			});
		} else {
			krtcClient.testLoudspeaker({
				playstatus: true,
				deviceId: meCurrentAudiooutputDeviceId,
				success: function () {
					setPaused(!paused)
				}
			});
		}
	}
	//关闭设备弹窗
	const closeDeviceChangeModal =() =>{
		var krtcClient = window.krtcClient
		krtcClient.testLoudspeaker({
			playstatus: true
		  });
		setVisible1(false)
	}
	//获取设备列表
	const getALLDevices = ()=>{
		const krtcClient = window.krtcClient;
		//麦克风列表
		krtcClient.listMicrophone({
			success: function(successModal) {
				const { data } = successModal
				console.log('--------------meAudioinputDevicesUpdate-----------------',data)
				setAudioInputDevices(data)
			},
			error: function(errModal) {
				const { description } = errModal
				message.warning(description)
			}
		})
		// 扬声器列表
		krtcClient.listLoudspeaker({
			success: function(successModal) {
				const { data } = successModal
				console.log('--------------meAudiooutputDevicesUpdate------------',data)
				setAudioOutDevices(data)
			},
			error: function(errModal) {
				const { description } = errModal
				message.warning(description)
			}
		})
		//摄像头列表
		krtcClient.listCamera({
			success: function(successModal) {
				const { data } = successModal
				console.log('----------------meVideoinputDevicesUpdate----------------',data)
				setVideoInputDevices(data)
			},
			error: function(errModal) {
				const { description } = errModal
				message.warning(description)
			}
		})
		//当前麦克风
		const currentMicrophone = krtcClient.getCurrentMicrophone()
		console.log("currentMicrophone",currentMicrophone)
		setMeCurrentAudioinputDeviceId(currentMicrophone)
		//当前扬声器
		const currentLoudspeaker = krtcClient.getCurrentLoudspeaker()
		console.log("currentLoudspeaker",currentLoudspeaker)
		setMeCurrentAudiooutputDeviceIdUpdate(currentLoudspeaker)
		//当前摄像头
		const currentCamera = krtcClient.getCurrentCamera()
		console.log("currentCamera",currentCamera)
		setMeCurrentVideoinputDeviceId(currentCamera)
	}
	//修改麦克风
	const changeMicrophone = (value)=>{
		console.log("--------修改麦克风----------",value)
		window.krtcClient.changeMicrophone({
			deviceId: value
		})
	}
	//修改扬声器
	const changeLoudspeaker = (value)=>{
		console.log("--------修改扬声器----------",value)
		window.krtcClient.changeLoudspeaker({
			deviceId: value
		})
	}
	//修改摄像头
	const changeCamera = (value)=>{
		console.log("--------修改摄像头----------",value)
		window.krtcClient.changeCamera({
			deviceId: value
		})
	}
	//退出会议删除
	const exitMeeting = ()=>{
		var krtcClient = window.krtcClient
			if(!krtcClient){ return }
	
			//本地视频流
			krtcClient.off('localVideoTrackUpdate', localVideoTrackUpdate)
	
			//成员列表
			krtcClient.off('mtlistPeersUpdate', handleMemberList)
			
	
			//普通消费者列表
			krtcClient.off('mediasConsumersUpdate', mediasConsumersUpdate)
	
			//双流消费者列表
			krtcClient.off('mediasShareConsumersUpdate',mediasShareConsumersUpdate)
	
			//我的声音
			krtcClient.off('meVolumeUpdate',meVolumeUpdate )
	
			//我的是否否静音
			krtcClient.off('meAudioMutedUpdate',meAudioMutedUpdate )
	
			//普通生产者
			krtcClient.off('mediasProducersUpdate',mediasProducersUpdate)
	
			//双流生产者
			krtcClient.off('mediasShareProducersUpdate',mediasShareProducersUpdate)

			//画面合成消费者
			krtcClient.off('mediasVmpConsumersUpdate',mediasVmpConsumersUpdate)

			//k
			krtcClient._singleSdk.off('error',handleErrorEvent)

	}
	const handleActivePeer = (_peer) => {
		let name = "";
		if(_peer.isDoubleflow){
			name = _peer.mtId + '-dualvideo';
		}else if(_peer.mediatype === 'confvmp'){
			name = _peer.mediatype + '-confvmp';
		}else{
			name = _peer.mtId + '-video'
		}
		setActivePeerKey(name);
	}
	
	return (
		<LayoutStyle className="m-layout">
			<div className="m-layout-header">
				<div className="m-layout-header-left">
					{meetingInfo &&meetingInfo.name && <div >{meetingInfo.name}</div>}
				</div>
				<div className="m-layout-header-right">
					{meInfo && (
						<div>
							用户名：{meInfo && meInfo.displayName} ({meInfo && meInfo.e164})
						</div>
					)}
				</div>
			</div>{' '}
			<div className="m-layout-body">
				<ViewStyle>
					<MainStyle>
						<MainCenterStyle ref={mainRef}></MainCenterStyle>
						<MainRightStyle
							className={classnames('main-right',"background", {
								'main-right-min': peerListView.length < 2 && activePeerKey
							})}
						>
							<div className="member_list_container">
								{peerListView.map((m, i) => {
									let track = null
									var key = ''
									let  mediatType = "video";
									if (m.isDoubleflow) {
										key = m.mtId + '-dualvideo'
										if (shareConsumers[m.mtId + '-dualvideo']) {
											track = shareConsumers[m.mtId + '-dualvideo'].track
											mediatType = 'dualvideo'
										}
									} else if(m.mediatype === 'confvmp'){
										key = m.mediatype + '-confvmp'
										mediatType ='confvmp'
										if (vmpConsumers[key]) {
											track = vmpConsumers[key].track
										}
									}else {
										key = m.mtId + '-video'
										if (consumersMap[m.mtId + '-video']) {
											track = consumersMap[m.mtId + '-video'].track
										}
									}


									const firstStyle = {
										width: getFitRect(width, height).width + 'px',
										height: getFitRect(width, height).height - 2 + 'px',
										objectFit: 'cover',
										margin: '0px',
										top: (height - getFitRect(width, height).height) / 2  + 'px',
										left: (width - getFitRect(width, height).width) / 2 + 'px',
										position: 'absolute'
									}
									
									const isMe = window.krtcClient.getUserId() === m.e164

									return (
										<>
											<PeerView
												key={i}
												onPeerClick={() => {
													handleActivePeer(m)
												}}
												mediatType={mediatType}
												track={isMe ? localVideoTrack : track}
												krtcClient={window.krtcClient}
												peerInfo={m}
												isMe={isMe}
												toBigView={key.length > 0 && key === activePeerKey}
												style={key.length > 0 && key === activePeerKey ? firstStyle : {}}
											/>
										</>
									)
								})}
							</div>
						</MainRightStyle>
						<MainRightStyle
							className={classnames('main-right', {
								'main-right-min': !detailExpand && !mtListShow,
								// 'member_list_container':mtListShow
							})}
						>
							{mtListShow && <MemberList memberList={mtList}></MemberList>}
							{detailExpand && <Detail meetingInfo={meetingInfo}></Detail>}
						</MainRightStyle>
					</MainStyle>
					<BToolStyle>
						<BToolLeftStyle>
							<div>当前大屏：{activePeerKey}</div>
						</BToolLeftStyle>
						<BToolMidStyle>
							<div className="meVolumeContainer">
								<div className="meVolumeInner" style={{ height: meVolumeValue * 10 + '%' }}></div>
							</div>
							<div
								className={classnames('tool-each', {
									on: meInfo && meInfo.micState === true,
									off: !(meInfo && meInfo.micState === true)
									
								})}
								onClick={() => {
									if (meInfo && meInfo.micState === true) {
										window.krtcClient.disableMicrophone({ isInd: true })
									} else {
										window.krtcClient.enableMicrophone({ isInd: true})
									}
								}}
							>
								<AudioOutlined/>

							</div>
							<div
								className={classnames('tool-each', {
									off: audioMuted,
									on: !audioMuted
								})}
								onClick={() => {
									if (audioMuted) {
										window.krtcClient.enableLoudspeaker()
									} else {
										window.krtcClient.disableLoudspeaker()
									}
								}}
							>
								<SoundOutlined />
							</div>

							{}

							<div
								className={classnames('tool-each', {
									off: !isVideoProducer,
									on: isVideoProducer
								})}
								onClick={() => {
									isVideoProducer ? window.krtcClient.disableCamera() : window.krtcClient.enableCamera()
								}}
							>
								<VideoCameraOutlined />
							</div>

							{/* <div className="tool-each">
						<FullscreenOutlined />
					</div> */}

							<div
								className={classnames('tool-each', {
									off: !isShareProducer,
									on: isShareProducer
								})}
								onClick={() => {
									isShareProducer ? window.krtcClient.disableShare() : window.krtcClient.enableShare()
								}}
							>
								<FundProjectionScreenOutlined />
							</div>

							<div
								className={classnames('tool-each', {
									off: !isShareProducer,
									on: isShareProducer
								})}
								onClick={() => {
									window.krtcClient.exit({success:()=>{
										exitMeeting()
										history.push("/list");
									}})
								}}
							>
								<PhoneOutlined />
							</div>
						</BToolMidStyle>
						<BToolRightStyle>
							<SettingFilled
								onClick={() => {
									getALLDevices()
									setVisible1(true)
								}}
							>
								设置
							</SettingFilled>
							<UserAddOutlined onClick={()=>{handleMtList()}}/>
							<MenuOutlined
								onClick={() => {
									handleDetail()
								}}
							/>
						</BToolRightStyle>
					</BToolStyle>
					<MemberListStyle></MemberListStyle>

					<Modal
						width={500}
						height={300}
						visible={visible1}
						title="设置"
						bodyStyle={{ padding: '10px 30px' }}
						onCancel={() => closeDeviceChangeModal()}
						footer={[
							<Button key="back" onClick={() => closeDeviceChangeModal()}>
								关闭
							</Button>
						]}
						destroyOnClose={false}
					>
						<Row style={{ marginBottom: '10px' }}>
							<Col span={4}>麦克风</Col>
							<Col span={20}>
								<Select
									defaultValue={meCurrentAudioinputDeviceId}
									style={{ width: '100%' }}
									onChange={changeMicrophone}
								>
									{audioInputDevices.map((device, index) => {
										return (
											<Select.Option value={device.deviceId} key={index}>
												{device.label}
											</Select.Option>
										)
									})}
								</Select>
								<VolumBarWrap className="volum-bar-wrap">
									<div className="volum-bar-outer">
									<div className="volum-bar-inner">
										<div className={classnames("volum-bar-bg", "volume" + meVolumeValue)} ></div>
									</div>
									</div>
								</VolumBarWrap>
							</Col>
						</Row>

						<Row style={{ marginBottom: '10px' }}>
							<Col span={4}>扬声器</Col>
							<Col span={15} style={{ paddingRight: '10px' }}>
								<Select
									defaultValue={meCurrentAudiooutputDeviceId}
									style={{ width: '100%' }}
									onChange={changeLoudspeaker}
								>
									{audioOutDevices.map((device, index) => {
										return (
											<Select.Option value={device.deviceId} key={index}>
												{device.label}
											</Select.Option>
										)
									})}
								</Select>
							</Col>
							<Col span={4} >
								<Button key="back" onClick={() => handleTestBtnClick()} >
									{paused ? "开始测试" : "停止测试"}
								</Button>
							</Col>
						</Row>

						<Row style={{ marginBottom: '10px' }}>
							<Col span={4}>视频输出</Col>
							<Col span={20}>
								<Select 
									defaultValue={meCurrentVideoinputDeviceId} 
									style={{ width: '100%' }}
									onChange={changeCamera}
								>
									{videoInputDevices.map((device, index) => {
										return (
											<Select.Option value={device.deviceId} key={index}>
												{device.label}
											</Select.Option>
										)
									})}
								</Select>

								<video
									autoPlay
									playsInline
									ref={testVideRef}
									style={{
										transform: !facemode ? 'rotateY(180deg)' : 'none',
										backgroundColor: '#000',
										width: '368px',
										height: '200px',
										objectFit: 'cover',
										marginTop: '10px'
									}}
								/>
								<div>
									<Checkbox checked={facemode} onClick={() => setFacemode(!facemode)} />{' '}
									启用本地摄像头镜像
								</div>
							</Col>
						</Row>
					</Modal>
				</ViewStyle>
			</div>
		</LayoutStyle>
	)
}

export default Room
